/*  readpipe.c
||
||  This OCI program demonstrates how to read messages from
||  a named pipe.  This program can be run in
||  a window or terminal session, and be used to print messages
||  from a PL/SQL source program (an anonymous block or a
||  stored procedure).
||
||  This is very useful in debugging PL/SQL programs.
||
||  First, you must create a PL/SQL package and package body that packs
||  your message(s) and sends them to a named pipe.
||  In the example below, two 'put' procedures are implemented,
||  overloaded by type: VARCHAR2 and NUMBER.  This you can extend
||  as required for additional types.
||  
||  Store this package in the server in an appropriate schema,
||  and grant execute privilege on it to the users who require it.
||
||  You will need to grant execute privilege on the dbms_pipe package
||  to the owner of the plsdbg package below.
||
||  Note that this example uses only a single named pipe.  It could
||  be confusing if more than one PL/SQL program were writing to
||  the same pipe (unless some identifying protocol is established).
||  It would certainly be confusing if several OCI programs like
||  the one below were reading the same pipe.
||
||  Here is the example package:
*/ 
create or replace package plsdbg as
  procedure put(info varchar2);
  procedure put(n number);
end plsdbg;
/
create or replace package body plsdbg as
  procedure put(info varchar2) is
    status integer;  -- ret. val. required for the 
                     -- send_message function, not used here
  begin
    dbms_pipe.pack_message(info);
    status := dbms_pipe.send_message('ora$plsql_debug');
  end;
-- 
  procedure put(n number) is
    chr    varchar2(44);
    status integer;
  begin
    chr := to_char(n);
    dbms_pipe.pack_message(chr);
    status :=  dbms_pipe.send_message('ora$plsql_debug');
  end;
end plsdbg;
/
/*
|| In a PL/SQL program, you call plsdbg to write messages to the
|| pipe 'ora$plsql_debug' like this:
*/
declare
     my_var   integer;
   begin
     my_var := 99;
     plsdbg.put('You have new message at ' || TO_CHAR(sysdate, 'MON-DD-YYYY HH24:MI:SS'));
end;
/
